import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import javax.swing.UIManager.*;

class GUIWindow extends JFrame implements ActionListener
{
    private JButton browseButton;
    private JButton browseButtonM;
    private JButton swapButton;
    private JButton restoreButton;
    private JButton testButton;
    private JTextField textArea;
    private JTextField textAreaM;
    
    public GUIWindow()
    {
        setTitle("MapCraft");
        setSize(400, 130);
        setLocation(10, 200);
        setResizable(false);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        
        // Hook the Window Close event so that we don't end up with swapped maps after we close.
        addWindowListener(
            new WindowAdapter() {
                public void windowClosing(WindowEvent winEvt) {
                    if( MapSwapper.getSingleton().swapped() )
                        MapSwapper.getSingleton().restore();
                }
            }
        );
        
        JPanel panel = new JPanel();
        panel.setLayout( new BorderLayout() );
        
        JTabbedPane tabbedPane = new JTabbedPane();
        panel.add( tabbedPane, BorderLayout.CENTER );
        
        // Create tab panels.
        JPanel singlePlayerPanel = new JPanel();
        singlePlayerPanel.setLayout( new BorderLayout() );
        
        JLabel jl = new JLabel("Map File: ");
        textArea = new JTextField(20);
        textArea.setEditable(false);
        browseButton = new JButton("Browse");
        browseButton.addActionListener(this);
        testButton = new JButton("Launch");
        testButton.addActionListener(this);
        
        singlePlayerPanel.add( jl, BorderLayout.WEST );
        singlePlayerPanel.add( textArea, BorderLayout.CENTER );
        singlePlayerPanel.add( browseButton, BorderLayout.EAST );
        singlePlayerPanel.add( testButton, BorderLayout.SOUTH );
        singlePlayerPanel.setOpaque(false);
        
        JPanel multiPlayerPanel = new JPanel();
        multiPlayerPanel.setLayout( new BorderLayout() );
        jl = new JLabel("Map File: ");
        textAreaM = new JTextField(20);
        textAreaM.setEditable(false);
        browseButtonM = new JButton("Browse");
        browseButtonM.addActionListener(this);
        swapButton = new JButton("Swap");
        swapButton.addActionListener(this);
        restoreButton = new JButton("Restore");
        restoreButton.setEnabled(false);
        restoreButton.addActionListener(this);
        
        multiPlayerPanel.add( jl, BorderLayout.WEST );
        multiPlayerPanel.add( textAreaM, BorderLayout.CENTER );
        multiPlayerPanel.add( browseButtonM, BorderLayout.EAST );
        
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridLayout());
        buttonPanel.setOpaque(false);
        buttonPanel.add( swapButton );
        buttonPanel.add( restoreButton );
        
        multiPlayerPanel.add( buttonPanel, BorderLayout.SOUTH );
        multiPlayerPanel.setOpaque(false);
        
        JPanel helpPanel = new JPanel();
        helpPanel.setLayout( new BorderLayout() );
        helpPanel.setOpaque(false);
        JTextArea helpText = new JTextArea();
        helpText.setEditable(false);
        helpText.setCursor(null);
        helpText.setOpaque(false);
        helpText.setFocusable(false);
        helpText.setWrapStyleWord(true);
        helpText.setLineWrap(true);
        helpText.setText("To play multiplayer, open Starcraft 2 and create a Battle.net game with Lost Temple as the map. " +
                         "Once everyone is in the lobby, have everyone hit Swap. Then play!");
        helpText.setFont( new Font("Monaco", 0, 12) );
        helpPanel.add(helpText, BorderLayout.CENTER );
        
        JPanel aboutPanel = new JPanel();
        aboutPanel.setLayout( new BorderLayout() );
        aboutPanel.setOpaque(false);
        JEditorPane aboutText = new JEditorPane();
        aboutText.setEditable(false); aboutText.setCursor(null); aboutText.setOpaque(false); aboutText.setFocusable(false); 
        aboutText.setContentType("text/rtf");
        aboutText.setFont( new Font("Monaco", 0, 12));
        //aboutText.setWrapStyleWord(true); aboutText.setLineWrap(true);
        aboutText.addHyperlinkListener( new EditorHyperlinkListener() );
        aboutText.setText("MapCraft was written by MotiveMe of SC2Mapster.com. Visit <a href=\"http://www.sc2mapster.com\">SC2Mapster.com</a> for your Starcraft 2 map resources.<br />View our project page <a href=\"http://www.sc2mapster.com/assets/mac-map-launcher/\">here</a>.");
        aboutPanel.add( aboutText, BorderLayout.CENTER );
        
        tabbedPane.addTab( "Single Player", singlePlayerPanel );
        tabbedPane.addTab( "Multiplayer", multiPlayerPanel );
        tabbedPane.addTab( "Help", helpPanel );
        tabbedPane.addTab( "About", aboutPanel );
        add( panel );
    }
    
    public void actionPerformed(ActionEvent e)
    {
        if( e.getSource() == browseButton )
        {
            // browse for things!
            JFileChooser fc = new JFileChooser();
            fc.addChoosableFileFilter( new SCMapFilter() );
            int ret = fc.showOpenDialog(new FileChooserImpl());
            if( ret == JFileChooser.APPROVE_OPTION )
            {
                File f = fc.getSelectedFile();
                String fileName = f.getAbsolutePath();
                textArea.setText(fileName);
            }
        } else if( e.getSource() == browseButtonM )
        {
            // browse for things!
            JFileChooser fc = new JFileChooser();
            fc.addChoosableFileFilter( new SCMapFilter() );
            int ret = fc.showOpenDialog(new FileChooserImpl());
            if( ret == JFileChooser.APPROVE_OPTION )
            {
                File f = fc.getSelectedFile();
                String fileName = f.getAbsolutePath();
                textAreaM.setText(fileName);
            }
        }
        else if( e.getSource() == swapButton )
        {
            if( textAreaM.getText().equals("") )
                return;
                
            swapButton.setEnabled(false);
            restoreButton.setEnabled(true);
            JOptionPane.showMessageDialog(this, "Be sure to press Restore once your game is complete!");
            
            MapSwapper.getSingleton().setMapFile( textAreaM.getText() );
            if( !MapSwapper.getSingleton().swap() )
            {
                JOptionPane.showMessageDialog(null, MapSwapper.getSingleton().getErrorText() );
            }
        }
        else if( e.getSource() == restoreButton )
        {
            swapButton.setEnabled(true);
            restoreButton.setEnabled(false);
            JOptionPane.showMessageDialog(this, "Your map has been restored!");
            
            if( !MapSwapper.getSingleton().restore() )
            {
                JOptionPane.showMessageDialog(null, MapSwapper.getSingleton().getErrorText() );
            }
        }
        else if( e.getSource() == testButton )
        {
            if( OperatingSystem.GetCurrentOS() == OperatingSystem.MACOSX )
            {
                String[] args = new String[5];
                args[0] = "open";
                args[1] = "/Applications/Starcraft II Beta/Starcraft II.app";
                args[2] = "--args";
                args[3] = "-run";
                args[4] = textArea.getText();
                try
                {
                    Runtime.getRuntime().exec(args);
                } catch (Exception ex) { }
            } else if( OperatingSystem.GetCurrentOS() == OperatingSystem.WINDOWS )
            {
                String executable = RegQuery.getStarcraftInstallPath() + "\\Starcraft II.exe";
                String[] args = new String[3];
                args[0] = executable;
                args[1] = "-run";
                args[2] = textArea.getText();
                try
                {
                    Runtime.getRuntime().exec(args);
                } catch (Exception ex) { }
            }
        }
    }
    
    public static void main(String[] args)
    {
        /* Enable Nimbus if not OS X 
         * default is ugly.
         */
        if( OperatingSystem.GetCurrentOS() != OperatingSystem.MACOSX )
        {
            try
            {
                for (LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
                    if ("Nimbus".equals(info.getName())) {
                        UIManager.setLookAndFeel(info.getClassName());
                        break;
                    }
                }
            } catch (Exception e) {
                // If Nimbus is not available, you can set the GUI to another look and feel.
            }
        }
        else
        {
            System.setProperty("com.apple.macos.smallTabs", "true");
        }
        
        if( !MapSwapper.getSingleton().hasSaveDirectory() )
        {
            JOptionPane.showMessageDialog(null, MapSwapper.getSingleton().getErrorText() );
            System.exit(-1);
        }
        
        if( !MapSwapper.getSingleton().hasLostTemple() )
        {
            JOptionPane.showMessageDialog(null, MapSwapper.getSingleton().getErrorText() );
            System.exit(-1);
        }
        
        
        SwingUtilities.invokeLater(new Runnable() {
        public void run() {
            JFrame f = new GUIWindow();
            f.setVisible(true);
        }
        });
    }
}