import java.io.*;

public class MapSwapper
{
    private final int m_currentOS = OperatingSystem.GetCurrentOS().ordinal();
    private final String m_blizzardCacheLocation;
    
    private final String m_cacheFile;
    
    private String mapFile;
    boolean m_swapped = false;

    private String m_errorText;
    
    private MapSwapper()
    {
        // Setup OS-dependent Map paths.
        if( m_currentOS == OperatingSystem.WINDOWS.ordinal() )
        {
            m_blizzardCacheLocation = OperatingSystem.GetApplicationDataDirectory() + "\\Blizzard Entertainment\\Battle.net\\Cache\\";
            m_cacheFile = "44/75/4475ddf83ebd5a0072d29a4009374cf2187f963acede493edfa5f6a5b07c4f49.s2ma";
        }
        else if( m_currentOS == OperatingSystem.MACOSX.ordinal() )
        {
            m_blizzardCacheLocation = OperatingSystem.GetApplicationDataDirectory() + "/Application Support/Battle.net/Cache/";
            m_cacheFile = "44/75/4475ddf83ebd5a0072d29a4009374cf2187f963acede493edfa5f6a5b07c4f49.s2ma";
        }
        else
        {
            m_blizzardCacheLocation = "";
            m_cacheFile = "";
            m_errorText = "Unsupported OS.";
        }
    }
    
    public void setMapFile(String s)
    {
        mapFile = s;
    }
    
    private void moveFile(String from, String to)
    {
        File f = new File(from);
        f.renameTo(new File(to));
    }
    
    public String getCacheFileName()
    {
        String name = m_cacheFile; 
        
        // Remove 44/75/ prefix or whatnot.
        int p = name.lastIndexOf('/');
        name = name.substring(p+1);
        return name;
    }
    
    public boolean swapped()
    {
        return m_swapped;
    }
    
    public boolean swap()
    {
        // abort if the move operation will fail.
        if( !hasLostTemple() || !hasSaveDirectory() )
            return false;
            
        moveFile( m_blizzardCacheLocation + m_cacheFile, OperatingSystem.GetDataDirectoryForOS() + getCacheFileName() );
        moveFile( mapFile, m_blizzardCacheLocation + m_cacheFile );
        m_swapped = true;
        return true;
    }
    
    public boolean restore()
    {
        // abort if the move operation will fail.
        if( !hasLostTemple() || !hasSaveDirectory() )
            return false;
            
        moveFile( m_blizzardCacheLocation + m_cacheFile, mapFile );
        moveFile( OperatingSystem.GetDataDirectoryForOS() + getCacheFileName(), m_blizzardCacheLocation + m_cacheFile );
        
        m_swapped = false;
        return true;
    }
    
    public boolean hasLostTemple()
    {
        // Find and check for existance of cacheFile.
        File f = new File( m_blizzardCacheLocation + m_cacheFile );
        m_errorText = "Unable to locate Lost Temple map at " + f.getAbsolutePath();
        return f.isFile();
    }
    public boolean hasSaveDirectory()
    {
        File f = new File( OperatingSystem.GetDataDirectoryForOS() );
        if( !f.isDirectory() ) // attempt to make our directory then.
        {
            if( f.mkdir() )
                return true;
            else
            {
                m_errorText = "Unable to create Application directory at: " + f.getAbsolutePath();
                return false;
            }
        }


        // Check for already-existed files, too.
        f = new File( OperatingSystem.GetDataDirectoryForOS() + getCacheFileName() );
        if( f.isFile() ) {
            // This file has to be deleted.
            if( f.delete() )
                return true;
            else
            {
                m_errorText = "Unable to delete pre-existing file at: " + f.getAbsolutePath();
                return false;
            }
        }
        
        return true;
    }

    public String getErrorText()
    {
        return m_errorText;
    }
    
    private static MapSwapper m_swapper;
    
    public static MapSwapper getSingleton()
    {
        if( m_swapper == null)
            m_swapper = new MapSwapper();
            
        return m_swapper;
    }
}